@@ -8,7 +8,7 @@ instypes <- c("t", "h", "n")
 # supported units of time
 units <- c("y", "q", "m", "w", "d", "h", "min", "s")
 # time zones for tests
-tzs <- intersect(OlsonNames(), c("Asia/Tokyo", "Europe/Warsaw",
+tzs <- intersect(.OlsonNames(), c("Asia/Tokyo", "Europe/Warsaw",
                                  "UTC", "Etc/GMT+1",
                                  "Europe/London", "America/New_York"))
 
